#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `robotica` package."""


def test_dummy():
    """Sample pytest test function with the pytest fixture as an argument."""
    assert True is not False
